/* ***************************** FILE HEADER **********************************
       Main Class File:  AlarmClockApp.java
       File:             AlarmClock.java
       
       Author:           Daniel Tomko
       Email:            dtomko@cs.wisc.edu
       Collaborators:    Used Jeffrey Shergalis' Time class from Lesson4
                         shergali@cs.wisc.edu

       Completion Date:  13 October 2000

       Course:           CS 302, Fall 2000
       TA:               Daniel Tomko

       Compiler:         CodeWarrior IDE 4.0 (JDK 1.2)
       Platform:         Windows NT 4.0
 **************************** 80 columns wide *********************************/

import java.util.*; // Calender

/**
 * This class implements the "circuit board" of an alarm clock. It performs the
 * functions of keeping track of the time of day and the set alarm time, as well
 * as incrementing the time each second and determining when the alarm should
 * "sound." In addition, based on the time of day for which the alarm is set,
 * it determines a message to be displayed on the Alarm interface object.
 *
 * Bugs: none known
 * @author Daniel Tomko dtomko@cs.wisc.edu
 *
**/

class AlarmClock {

  private static final int SECONDS_BOUND = 60;
  private static final int MINUTES_BOUND = 60;
  private static final int HOURS_BOUND   = 24;

  // ********************
  // *** DATA MEMBERS ***
  // ********************

  private
  String name; // the name of the user

  private
  Time hours,
       minutes,
       seconds;

  private
  int alarm_on_hours,
      alarm_on_minutes,
      alarm_on_seconds;

  private
  int alarm_off_hours,
      alarm_off_minutes,
      alarm_off_seconds;

  private
  boolean alarming; // true if the alarm is sounding

  private
  boolean alarm_set; // true if an alarm time is set

  // ********************
  // *** CONSTRUCTORS ***
  // ********************

  /**
   * Default Constructor (no parameters)
   *
   */

  public AlarmClock() {
    // call the other constructor
    this("no name");
  }

  /**
   * Constructor to do the real work.
   *
   * @param user_name The name to display on the clock.
   */

  public AlarmClock(String user_name) {

    name = user_name;

    // these will be initialized by the setClock method
    hours   = new Time(12, 0, HOURS_BOUND-1);
    minutes = new Time(0,  0, MINUTES_BOUND-1);
    seconds = new Time(0,  0, SECONDS_BOUND-1);

    // these will be set by the setAlarm method
    alarm_set         = false;
    alarming          = false;
    alarm_on_hours    = 0;
    alarm_on_minutes  = 0;
    alarm_on_seconds  = 0;
    alarm_off_hours   = 0;
    alarm_off_minutes = 0;
    alarm_off_seconds = 0;
  }

  // ************************
  // *** ACCESSOR METHODS ***
  // ************************

  /**
   * Returns true if the alarm is set, false otherwise.
   *
   *
   * @return boolean true if the alarm is set, false otherwise
   */

  public boolean isAlarmSet() {
    return alarm_set;
  }

  /**
   * Returns true if the alarm should be sounding at the current time.
   *
   *
   * @return boolean true if the alarm should be sounding at the current time
   */

  public boolean signalActive() {
    return alarming;
  }

  /**
   * Returns a string representation of the current time.
   *
   * @return A string representation of the current time.
   */

  public String toString() {
    return   hours.getValue() + ":"
           + minutes.getValue() + ":"
           + seconds.getValue();
  }

  /**
   * Returns a string representation of the current time.
   *
   * @return A message to display on the GUI this second
   */

  public String getMessage() {
    String message = name;
    
    if (alarming) {
      int current_hour = hours.getValue();
      if (current_hour >= 6 && current_hour < 12) {
        message = "Good Morning!";
      } else if (current_hour >= 12 && current_hour < 18) {
        message = "Good Afternoon!";
      } else if (current_hour >= 18 && current_hour < 20) {
        message = "Good Afternoon!";
      } else {
        message = "Good Night!";
      }
    }

    return message;
  }

  // ***********************
  // *** MUTATOR METHODS ***
  // ***********************

  /**
   * Sets the clock to a user specified time.
   *
   * @param hours  Starting  hours
   * @param minutes Starting minutes
   * @param seconds Starting seconds
   */

  public void setClock(int h, int m, int s) {
    // some error checking
    h = Math.abs(h);
    m = Math.abs(m);
    s = Math.abs(s);

    m += s / SECONDS_BOUND;
    s %= SECONDS_BOUND;
    h += m / MINUTES_BOUND;
    m %= MINUTES_BOUND;
    h %= HOURS_BOUND;

    hours   = new Time(h, 0, HOURS_BOUND-1); // Reminder: initial, min, max
    minutes = new Time(m, 0, MINUTES_BOUND-1);
    seconds = new Time(s, 0, SECONDS_BOUND-1);
  }

  /**
   * Sets the clock to the current system time.
   *
   */

  public void setClock() {
    // Get a Calender initialized to the current system time
    Calendar cal = Calendar.getInstance();

    // get current HOURS_BOUND hour time from system
    int h = cal.get(Calendar.HOUR);
    if (cal.get(Calendar.AM_PM) == Calendar.PM) {
      h += 12; // this is PM so convert to HOURS_BOUND hour clock
    }
    int m = cal.get(Calendar.MINUTE);
    int s = cal.get(Calendar.SECOND);

    // create time objects
    hours   = new Time(h, 0, HOURS_BOUND-1);
    minutes = new Time(m, 0, MINUTES_BOUND-1);
    seconds = new Time(s, 0, SECONDS_BOUND-1);
  }

  /**
   * Sets the alarm time and duration.
   *
   * @param hours The alarm hour (HOURS_BOUND hour hime)
   * @param minutes The alarm minute
   * @param seconds The alarm second
   * @param duration The length of time the alarm should "sound"
   */

  public void setAlarm(int hours, int minutes, int seconds, int duration) {

    if (duration == 0) {
      // if the duration is zero, the alarm should not go off so don't set it
      return;
    }

    // some error checking
    hours    = Math.abs(hours);
    minutes  = Math.abs(minutes);
    seconds  = Math.abs(seconds);
    duration = Math.abs(duration);

    minutes += seconds / SECONDS_BOUND;
    seconds %= SECONDS_BOUND;
    hours   += minutes / MINUTES_BOUND;
    minutes %= MINUTES_BOUND;
    hours   %= HOURS_BOUND;

    // set the alarm time
    alarm_off_hours   = alarm_on_hours   = hours;
    alarm_off_minutes = alarm_on_minutes = minutes;
    alarm_off_seconds = alarm_on_seconds = seconds;

    // compute the alarm off time
    alarm_off_seconds += duration;
    if (alarm_off_seconds >= SECONDS_BOUND) {
      alarm_off_minutes += alarm_off_seconds / SECONDS_BOUND;
      alarm_off_seconds %= SECONDS_BOUND;
    }
    if (alarm_off_minutes >= MINUTES_BOUND) {
      alarm_off_hours += alarm_off_minutes / MINUTES_BOUND;
      alarm_off_minutes %= MINUTES_BOUND;
    }
    alarm_off_hours %= HOURS_BOUND;

    alarm_set = true; // the alarm is on
  }

  /**
   * Increments the time by one second. Handles rollover of units.
   *
   */

  public void tick() {
    if (seconds.increment()) {
      if (minutes.increment()) {
        hours.increment();
      }
    }

    // should I turn the alarm on or off?
    if (alarm_set && !alarming) {
      if (seconds.getValue() == alarm_on_seconds &&
          minutes.getValue() == alarm_on_minutes &&
            hours.getValue() == alarm_on_hours) {
        alarming = true;
      }
    } else if (alarm_set && alarming) {
      if (seconds.getValue() == alarm_off_seconds &&
          minutes.getValue() == alarm_off_minutes &&
            hours.getValue() == alarm_off_hours) {
        alarming = false;
      }
    }
  }

  // ************************
  // *** TEST MAIN METHOD ***
  // ************************

  /**
   * Test driver main method to run a hard test on the AlarmClock class.
   *
   * @param argv Command line arguments (not used).
   */

  public static void main(String[] argv) {
    AlarmClock ac = new AlarmClock();
    int i;
    boolean pass;

    // test to see if an alarm at 23:59:55 for ten seconds works
    ac.setClock(23, 59, 54);
    ac.setAlarm(23, 59, 55, 10);

    pass = true;
    for (i=0; i<11; i++) {
      switch (i) {
        case 0:
          pass &= ac.signalActive() == false;
          break;
        case 1:
          pass &= ac.signalActive() == true;
          break;
        case 6:
          pass &= ac.signalActive() == true;
          pass &= ac.toString().equals("0:0:0");
          break;
        case 10:
          pass &= ac.signalActive() == true;
          pass &= ac.toString().equals("0:0:4");
          break;
        case 11:
          pass &= ac.signalActive() == false;
          pass &= ac.toString().equals("0:0:5");
          break;
        default:
          break; // do nothing
      }
      ac.tick();
    }
    if (pass) {
      System.out.println("Alarm rollover test passed.\n");
    } else {
      System.out.println("Alarm rollover test failed.\n");
    }
  }

} // END CLASS

